@@ -12,7 +12,7 @@
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * version 2 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -68,33 +68,31 @@ typedef void (*XdgMimeDestroy)  (void *user_data);
 #define xdg_mime_register_reload_callback     XDG_ENTRY(register_reload_callback)
 #define xdg_mime_remove_callback              XDG_ENTRY(remove_callback)
 #define xdg_mime_type_unknown                 XDG_ENTRY(type_unknown)
-#define xdg_mime_type_empty                   XDG_ENTRY(type_empty)
-#define xdg_mime_type_textplain               XDG_ENTRY(type_textplain)
 #define xdg_mime_get_icon                     XDG_ENTRY(get_icon)
 #define xdg_mime_get_generic_icon             XDG_ENTRY(get_generic_icon)
 
-#define _xdg_mime_mime_type_equal             XDG_RESERVED_ENTRY(mime_type_equal)
 #define _xdg_mime_mime_type_subclass          XDG_RESERVED_ENTRY(mime_type_subclass)
-#define _xdg_mime_unalias_mime_type           XDG_RESERVED_ENTRY(unalias_mime_type)  
+#define _xdg_mime_mime_type_equal             XDG_RESERVED_ENTRY(mime_type_equal)
+#define _xdg_mime_unalias_mime_type           XDG_RESERVED_ENTRY(unalias_mime_type)
 #endif
 
 extern const char xdg_mime_type_unknown[];
-extern const char xdg_mime_type_empty[];
-extern const char xdg_mime_type_textplain[];
 #define XDG_MIME_TYPE_UNKNOWN xdg_mime_type_unknown
-#define XDG_MIME_TYPE_EMPTY xdg_mime_type_empty
-#define XDG_MIME_TYPE_TEXTPLAIN xdg_mime_type_textplain
 
 const char  *xdg_mime_get_mime_type_for_data       (const void *data,
 						    size_t      len,
 						    int        *result_prio);
+#ifdef NOT_USED_IN_GIO
 const char  *xdg_mime_get_mime_type_for_file       (const char *file_name,
                                                     struct stat *statbuf);
 const char  *xdg_mime_get_mime_type_from_file_name (const char *file_name);
+#endif
 int          xdg_mime_get_mime_types_from_file_name(const char *file_name,
 						    const char *mime_types[],
 						    int         n_mime_types);
+#ifdef NOT_USED_IN_GIO
 int          xdg_mime_is_valid_mime_type           (const char *mime_type);
+#endif
 int          xdg_mime_mime_type_equal              (const char *mime_a,
 						    const char *mime_b);
 int          xdg_mime_media_type_equal             (const char *mime_a,
@@ -106,20 +104,24 @@ int          xdg_mime_mime_type_subclass           (const char *mime_a,
    * instead, but notice that that function expects you to free
    * the array it returns. 
    */
+#ifdef NOT_USED_IN_GIO
 const char **xdg_mime_get_mime_parents		   (const char *mime);
+#endif
 char **      xdg_mime_list_mime_parents		   (const char *mime);
 const char  *xdg_mime_unalias_mime_type		   (const char *mime);
 const char  *xdg_mime_get_icon                     (const char *mime);
 const char  *xdg_mime_get_generic_icon             (const char *mime);
 int          xdg_mime_get_max_buffer_extents       (void);
 void         xdg_mime_shutdown                     (void);
+#ifdef NOT_USED_IN_GIO
 void         xdg_mime_dump                         (void);
+#endif
 int          xdg_mime_register_reload_callback     (XdgMimeCallback  callback,
 						    void            *data,
 						    XdgMimeDestroy   destroy);
+#ifdef NOT_USED_IN_GIO
 void         xdg_mime_remove_callback              (int              callback_id);
-
-void xdg_mime_set_dirs (const char * const *dirs);
+#endif
 
    /* Private versions of functions that don't call xdg_mime_init () */
 int          _xdg_mime_mime_type_equal             (const char *mime_a,
