@@ -123,7 +123,7 @@ _g_param_type_init (void)
 
   /* This should be registred as GParamSpec instead of GParam, for
    * consistency sake, so that type name can be mapped to struct name,
-   * However, some language bindings, most noticable the python ones
+   * However, some language bindings, most noticeable the python ones
    * depends on the "GParam" identifier, see #548689
    */
   type = g_type_register_fundamental (G_TYPE_PARAM, g_intern_static_string ("GParam"), &param_spec_info, &finfo, G_TYPE_FLAG_ABSTRACT);
@@ -173,9 +173,6 @@ g_param_spec_finalize (GParamSpec *pspec)
 {
   g_datalist_clear (&pspec->qdata);
 
-  if (!(pspec->flags & G_PARAM_STATIC_NAME))
-    g_free (pspec->name);
-  
   if (!(pspec->flags & G_PARAM_STATIC_NICK))
     g_free (pspec->_nick);
 
@@ -275,6 +272,9 @@ g_param_spec_ref_sink (GParamSpec *pspec)
  *
  * Get the name of a #GParamSpec.
  *
+ * The name is always an "interned" string (as per g_intern_string()).
+ * This allows for pointer-value comparisons.
+ *
  * Returns: the name of @pspec.
  */
 const gchar *
@@ -428,9 +428,15 @@ g_param_spec_internal (GType        param_type,
     }
   else
     {
-      pspec->name = g_strdup (name);
-      canonicalize_key (pspec->name);
-      g_intern_string (pspec->name);
+      if (is_canonical (name))
+        pspec->name = (gchar *) g_intern_string (name);
+      else
+        {
+          gchar *tmp = g_strdup (name);
+          canonicalize_key (tmp);
+          pspec->name = (gchar *) g_intern_string (tmp);
+          g_free (tmp);
+        }
     }
 
   if (flags & G_PARAM_STATIC_NICK)
@@ -911,7 +917,7 @@ g_param_spec_pool_insert (GParamSpecPool *pool,
 			  GParamSpec     *pspec,
 			  GType           owner_type)
 {
-  gchar *p;
+  const gchar *p;
   
   if (pool && pspec && owner_type > 0 && pspec->owner_type == 0)
     {
@@ -989,26 +995,31 @@ param_spec_ht_lookup (GHashTable  *hash_table,
 
   if (!pspec && !is_canonical (param_name))
     {
+      gchar *canonical;
+
+      canonical = g_strdup (key.name);
+      canonicalize_key (canonical);
+
       /* try canonicalized form */
-      key.name = g_strdup (param_name);
+      key.name = canonical;
       key.owner_type = owner_type;
-      
-      canonicalize_key (key.name);
+
       if (walk_ancestors)
-	do
-	  {
-	    pspec = g_hash_table_lookup (hash_table, &key);
-	    if (pspec)
-	      {
-		g_free (key.name);
-		return pspec;
-	      }
-	    key.owner_type = g_type_parent (key.owner_type);
-	  }
-	while (key.owner_type);
+        do
+          {
+            pspec = g_hash_table_lookup (hash_table, &key);
+            if (pspec)
+              {
+                g_free (canonical);
+                return pspec;
+              }
+            key.owner_type = g_type_parent (key.owner_type);
+          }
+        while (key.owner_type);
       else
-	pspec = g_hash_table_lookup (hash_table, &key);
-      g_free (key.name);
+        pspec = g_hash_table_lookup (hash_table, &key);
+
+      g_free (canonical);
     }
 
   return pspec;
@@ -1304,7 +1315,7 @@ g_param_spec_pool_list (GParamSpecPool *pool,
 }
 
 
-/* --- auxillary functions --- */
+/* --- auxiliary functions --- */
 typedef struct
 {
   /* class portion */
@@ -1406,7 +1417,7 @@ g_param_type_register_static (const gchar              *name,
 /**
  * g_value_set_param:
  * @value: a valid #GValue of type %G_TYPE_PARAM
- * @param: the #GParamSpec to be set
+ * @param: (allow-none): the #GParamSpec to be set
  *
  * Set the contents of a %G_TYPE_PARAM #GValue to @param.
  */
@@ -1428,7 +1439,7 @@ g_value_set_param (GValue     *value,
 /**
  * g_value_set_param_take_ownership: (skip)
  * @value: a valid #GValue of type %G_TYPE_PARAM
- * @param: the #GParamSpec to be set
+ * @param: (allow-none): the #GParamSpec to be set
  *
  * This is an internal function introduced mainly for C marshallers.
  *
@@ -1444,7 +1455,7 @@ g_value_set_param_take_ownership (GValue     *value,
 /**
  * g_value_take_param: (skip)
  * @value: a valid #GValue of type %G_TYPE_PARAM
- * @param: the #GParamSpec to be set
+ * @param: (allow-none): the #GParamSpec to be set
  *
  * Sets the contents of a %G_TYPE_PARAM #GValue to @param and takes
  * over the ownership of the callers reference to @param; the caller
